/*----------------------------------------------------------*/
/* SAS Global Forum code example for presentation 5244-2016 */
/* Authors: Stacey Christian, Michael Whitcher,             */
/*          Don McAlister, Phil Hanna,                      */
/* Example by: Katherine Taylor                             */
/*                                                          */
/* Executed using a Windows 64bit system                    */
/*                                                          */
/* Notes: This program auto-generates data that is later    */
/* used to construct a HPRISK cube.  The cube is then used  */
/* with the HPR reader to give example of how to perform    */
/* ad hoc DS2 query processing.                             */
/*----------------------------------------------------------*/

/*01 Create libraries */
%let rootdir = \\location\to\demo_incrementalriskcharge\;
options dlcreatedir;

libname rdenv "&rootdir.RDEnvironment";
libname inputs "&rootdir.InputData";
libname results "&rootdir.RDResults";
%let EnvName = incrementalriskcharge;

/* Modify to adjust the size of the HPRISK cube to be generated */
%let numInstruments = 100;
%let numHorizons = 5;
%let numStates = 100;


/* Create input data */
data inputs.interest_rates;
   input rate1_1 rate1_2 rate1_3 rate1_4 rate1_5 rate1_6 rate1_7 rate1_8 rate1_9
         rate2_1 rate2_2 rate2_3 rate2_4 rate2_5 rate2_6 rate2_7 rate2_8 rate2_9
         rate3_1 rate3_2 rate3_3 rate3_4 rate3_5 rate3_6 rate3_7 rate3_8 rate3_9
         rate4_1 rate4_2 rate4_3 rate4_4 rate4_5 rate4_6 rate4_7 rate4_8 rate4_9
         rate5_1 rate5_2 rate5_3 rate5_4 rate5_5 rate5_6 rate5_7 rate5_8 rate5_9
         rate6_1 rate6_2 rate6_3 rate6_4 rate6_5 rate6_6 rate6_7 rate6_8 rate6_9
         rate7_1 rate7_2 rate7_3 rate7_4 rate7_5 rate7_6 rate7_7 rate7_8 rate7_9
         rate8_1 rate8_2 rate8_3 rate8_4 rate8_5 rate8_6 rate8_7 rate8_8 rate8_9;
   datalines;
0.0234  0.02435 0.0253  0.02635 0.0274  0.0281  0.0288  0.0295  0.0302
0.0251  0.02605 0.027   0.02805 0.0291  0.0298  0.0305  0.03115 0.0318
0.029   0.03    0.031   0.032   0.033   0.03365 0.0343  0.035   0.0357
0.0333  0.03435 0.0354  0.03645 0.0375  0.0382  0.0389  0.03965 0.0404
0.03765 0.0387  0.03975 0.0408  0.04185 0.04255 0.04325 0.044   0.04475
0.042   0.04305 0.0441  0.04515 0.0462  0.0469  0.0476  0.04835 0.0491
0.04635 0.0474  0.04845 0.0495  0.05055 0.05125 0.05195 0.0527  0.05345
0.0507  0.05175 0.0528  0.05385 0.0549  0.0556  0.0563  0.05705 0.0578
;
run;
options validvarname=any;
data inputs.transmat;
   input _NAME_ $ AAA 'AA+'n A BBB 'BB-'n B CCC Default;
   datalines;
AAA     0.9081  0.0833  0.0068  0.0006  0.0012  0       0       0
AA+     0.007   0.9065  0.0779  0.0064  0.0006  0.0014  0.0002  0
A       0.0009  0.0227  0.9105  0.0552  0.0074  0.0026  0.0001  0.0006
BBB     0.0002  0.0033  0.0595  0.8693  0.053   0.0117  0.0012  0.0018
BB-     0.0003  0.0014  0.0067  0.0773  0.8053  0.0884  0.01    0.0106
B       0       0.0011  0.0024  0.0043  0.0648  0.8346  0.0407  0.0521
CCC     0.0022  0       0.0022  0.013   0.0238  0.1124  0.6486  0.1978
Default 0       0       0       0       0       0       0       1
;

data inputs.instdata;
length industry $32;
insttype = "CouponBond";
array initratings{8} $ _temporary_ ("AAA" "AA+" "A" "BBB" "BB-" "B" "CCC" "Default");
array industryarray{5} $32 _temporary_ ("Health" "Energy" "Financial" "Government" "Consumer");


do i = 1 to &numInstruments;
instnum = put(i,z4.);
instid = compress("InstID_"||instnum);
face_value = round(100 + ceil(ranuni(321)*500),10);
coupon_rate = 0.5;
years = 5;
ret = 1.5;
initial_rating_num = rand("Table", 0.05, .20, .40, .15, .1, 0.05, 0.05);
initial_rating = initratings[initial_rating_num];

industry_num = rand("Table",.1,.1,.3,.4,.1);
industry = industryarray[industry_num] ;


output;
end;
drop instnum i industry_num;
run;

data inputs.current;
   set inputs.interest_rates;
   ret = 1.5;
   output;
run;

data inputs.scendata;
do i = 1 to &numHorizons;
_HORIZON_ = i;
do j = 1 to &numStates;
ret = 1;
output;
end;
end;
drop i j;
run;




%macro declareRates();
           %do j=1 %to 8;
              %if &j^=8 %then %do;
                 %do i=1 %to 9;
                    rate&j._&i num ir currency=usd maturity=&i year category="IR",
                 %end;
              %end;
              %else %do;
                 %do i=1 %to 9-1;
                    rate&j._&i num ir currency=usd maturity=&i year category="IR",
                 %end;
                 rate&j._9 num ir currency=usd maturity=&i year category="IR"
              %end;
           %end;
%mend declareRates;

%macro declareArrays();
 array AAA_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate1_&i
         %end;)
         label="AAA interest rate curve";

   array AA_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate2_&i
         %end;)
         label="AA interest rate curve";

   array A_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate3_&i
         %end;)
         label="A interest rate curve";

   array BBB_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate4_&i
         %end;)
         label="BBB interest rate curve";

   array BB_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate5_&i
         %end;)
         label="BB interest rate curve";

   array B_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate6_&i
         %end;)
         label="B interest rate curve";

   array CCC_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate7_&i
         %end;)
         label="CCC interest rate curve";

   array Default_curve ir currency=usd category=IR
         elements=(
         %do i=1 %to 9;
            rate8_&i
         %end;)
         label="Default interest rate curve";

%mend declareArrays;


/* 02 Create RD Environment */
proc risk;

   env new = rdenv.&EnvName;

   declare instvars = (
           face_value num var,
           coupon_rate num var,
           years num var,
           initial_rating_num num var,
         initial_rating char 32 var,
         industry char 32 var
           );

   declare riskfactors=(
           ret num var mlevel=interval label="firm return",
         %declareRates);

  %declareArrays

  declare outvars = (
     end_rating num computed,
      start_rating num computed,
      credit_movement num computed,
      credit_migration_loss num computed,
      _value_no_creditchange num computed,
      bond_initial_rating num computed,
      cumulative_credit_migration_loss num computed);

  marketdata CurrentMarket
              file = inputs.current
              type = current
              label = "Current market states";

  marketdata ScenFile
              file = inputs.scendata
              type = scenarios
              interval = year
              label = "Scenario data";

   transmat mytransmat data=inputs.transmat type=initial cdf=mycdf interval=year;
   env save;

run;

proc compile outlib = rdenv.&EnvName
             env = rdenv.&EnvName
             package = rskprc;

   function mycdf(x);
      y=cdf('NORMAL',x);
      return(y);
   endsub;

   function pvcf(amount,c_rate,yrs,ircurv[*]) kind=price;
      coupon = amount*c_rate/2;
      prc = 0;
      do i=1 to 9;
         prc = prc+coupon/(1+ircurv{i}/2)**i;
      end;
      prc = prc + amount/(1+ircurv{9}/2)**9;
      return(prc);
   endsub;

   method bond_pricing kind=price;
   beginblock main;

/*      put instID= simulationtime= simulationreplication= simulationHorizon= _date_ date9.;*/
      if simulationtime < 1 then do; start = 0; current_rating =initial_rating_num; new_rating = initial_rating_num; end;
      else do;
         if simulationtime = 1 then current_rating = initial_rating_num; else current_rating = new_rating;
       randret = rannor(321);
       call get_rating(mytransmat,current_rating,randret,.,new_rating);

      end;
      array ircurv{9};
      do i=1 to 9;
         select;
            when (new_rating=1) ircurv{i} = AAA_curve{i};
            when (new_rating=2) ircurv{i} = AA_curve{i};
            when (new_rating=3) ircurv{i} = A_curve{i};
            when (new_rating=4) ircurv{i} = BBB_curve{i};
            when (new_rating=5) ircurv{i} = BB_curve{i};
            when (new_rating=6) ircurv{i} = B_curve{i};
            when (new_rating=7) ircurv{i} = CCC_curve{i};
            when (new_rating=8) ircurv{i} = Default_curve{i};
         end;
      end;
     nrat = current_rating+ret-ret;
     bond_initial_rating = initial_rating_num;
     start_rating = current_rating;
     end_rating = new_rating;
     credit_movement = -1 * (new_rating-current_rating); /* So B to CCC would be -1 */
      _value_ = pvcf(face_value,coupon_rate,years,ircurv);
      do i=1 to 9;
         select;
            when (start_rating=1) ircurv{i} = AAA_curve{i};
            when (start_rating=2) ircurv{i} = AA_curve{i};
            when (start_rating=3) ircurv{i} = A_curve{i};
            when (start_rating=4) ircurv{i} = BBB_curve{i};
            when (start_rating=5) ircurv{i} = BB_curve{i};
            when (start_rating=6) ircurv{i} = B_curve{i};
            when (start_rating=7) ircurv{i} = CCC_curve{i};
            when (start_rating=8) ircurv{i} = Default_curve{i};
         end;
      end;
     _value_no_creditchange = pvcf(face_value,coupon_rate,years,ircurv);
     credit_migration_loss = _value_no_creditchange - _value_;
      do i=1 to 9;
         select;
            when (bond_initial_rating=1) ircurv{i} = AAA_curve{i};
            when (bond_initial_rating=2) ircurv{i} = AA_curve{i};
            when (bond_initial_rating=3) ircurv{i} = A_curve{i};
            when (bond_initial_rating=4) ircurv{i} = BBB_curve{i};
            when (bond_initial_rating=5) ircurv{i} = BB_curve{i};
            when (bond_initial_rating=6) ircurv{i} = B_curve{i};
            when (bond_initial_rating=7) ircurv{i} = CCC_curve{i};
            when (bond_initial_rating=8) ircurv{i} = Default_curve{i};
         end;
      end;

     start = start + pvcf(face_value,coupon_rate,years,ircurv) - _value_;
      cumulative_credit_migration_loss = start;
     endblock;
   endmethod;

run;

/* Set up and Run analysis */
proc risk;

   env open = rdenv.&EnvName;

   /* Instrument Types */
   instrument CouponBond
              label = "bond instruments"
              variables = (face_value coupon_rate years initial_rating_num initial_rating)
              methods = (price bond_pricing);

   /* Portfolio source to portfolio file */
   instdata instdata file=inputs.instdata format=simple;
   sources source instdata;
   read sources=source out=read;

   /* Analysis and project */
   simulation Scenrat
              label = "Using scenario simulation"
              method = scenario
              interval = year
              data = ScenFile
              match = horizon
              horizon = (1 2 3 4 5);

   project Scenbond
           data = CurrentMarket
           analysis = Scenrat
           portfolio = read
           outlib = results
           options = (outall) ;

   env save;
run;
proc risk;

   env open = rdenv.&EnvName;
   runproject Scenbond
              options = (
              alpha = 0.05
              allprice
              states
           simstat
              );

run;
