/*----------------------------------------------------------*/
/* SAS Global Forum code example for presentation 5244-2016 */
/* Authors: Stacey Christian, Michael Whitcher,             */
/*          Don McAlister, Phil Hanna,                      */
/* Example by: Katherine Taylor                             */
/*                                                          */
/* Executed using a Windows 64bit system                    */
/*                                                          */
/* Notes: This program auto-generates data that is later    */
/* used to construct a HPRISK cube.  The cube is then used  */
/* with the HPR reader to give example of how to perform    */
/* ad hoc DS2 query processing.                             */
/*----------------------------------------------------------*/

/* Run HPRisk */

/* Setup for cubetotable registration */
%include "cubereader_setup.sas";


/*01 Create libraries */

%let rootdir = \\location\to\demo_incrementalriskcharge\;
options dlcreatedir;

libname rdenv   "&rootdir.RDEnvironment";
libname inputs  "&rootdir.InputData";
libname results "&rootdir.RDResults";
libname hpres   "&rootdir.HPResults";

%let EnvName = incrementalriskcharge;

/* Modify to adjust the size of the HPRISK cube to be generated */
%let numInstruments = 100;
%let numHorizons = 5;
%let numStates = 100;


data inputs.hpinstdata;
length industry $32;
insttype = "CouponBond";
array initratings{8} $ _temporary_ ("AAA" "AA+" "A" "BBB" "BB-" "B" "CCC" "Default");
array industryarray{5} $32 _temporary_ ("Health" "Energy" "Financial" "Government" "Consumer");


do i = 1 to &numInstruments;
instnum = put(i,z4.);
instid = compress("InstID_"||instnum);
face_value = round(100 + ceil(ranuni(321)*500),10);
coupon_rate = 0.5;
years = 5;
ret = 1.5;
initial_rating_num = rand("Table", 0.05, .20, .40, .15, .1, 0.05, 0.05);
initial_rating = initratings[initial_rating_num];

industry_num = rand("Table",.1,.1,.3,.4,.1);
industry = industryarray[industry_num] ;


output;
end;
drop instnum i industry_num;
run;

data inputs.hpstatesdata;
set results.states_v(obs=1);
orig_date = _date_;
length AnalyisisName $ 12;
AnalyisisName = "BASECASE";
output;
AnalyisisName = "ScenRating";
do i = 1 to &numStates;
   SimulationReplication = i;
   do j = 1 to &numHorizons;
       SimulationTime= j;
      simulationhorizon = SimulationTime;
      _date_ = intnx('YEAR', orig_date, SimulationTime, 's');
      output;
   end;
end;
drop i j statenumber analysisnumber analysispart orig_date;
run;

%let nodes = 5;
%let threads = 10;
%let gridpath=/local/data/risk/;
%let cubesuffix=demo; /*this is just appended to the cube name, set as you'd like */

proc hpexport
expprojlib = work
env = RDEnv.&EnvName;
run;

proc hprisk
task = clean
cube = "&rootdir.&envname._&cubesuffix";
run;
quit;

proc hprisk debug
   expprojlib       = work
   tracelog         = yes
   task             = RUNPROJECT
   cube             = "&rootdir.&envname._&cubesuffix"
   filesprefix      = "&gridpath.&envname._&cubesuffix"
   statesdata       = inputs.hpstatesdata
   instdata         = inputs.hpinstdata
   outallprice      = hpres.hpallprice
   priceby          = positions
   cube_store_type  = hdfs
   hdfs_cube_copies = 2

   ;
   performance nodes=&nodes nthreads=&threads;
   crossclassvars industry insttype instid;
   outvars add = (cumulative_credit_migration_loss bond_initial_rating end_rating);

   cubetotable table=anyfiler.&envname replace content=prices;
run;
quit;

/* added a sort here to make sure allprice is in the order that it will be when using the cube reader */
proc sort data = hpres.hpallprice;
    by InstId SimulationReplication SimulationTime;
run;